"""Product block for :class:`Super PoP Switch` products."""

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle

from gso.products.product_blocks.site import (
    SiteBlock,
    SiteBlockInactive,
    SiteBlockProvisioning,
)
from gso.utils.shared_enums import IPv4AddressType, PortNumber, Vendor


class SuperPopSwitchBlockInactive(
    ProductBlockModel,
    lifecycle=[SubscriptionLifecycle.INITIAL],
    product_block_name="SuperPopSwitchBlock",
):
    """A Super PoP switch that's being currently inactive. See :class:`SuperPopSwitchBlock`."""

    super_pop_switch_fqdn: str | None = None
    super_pop_switch_ts_port: PortNumber | None = None
    super_pop_switch_mgmt_ipv4_address: IPv4AddressType | None = None
    super_pop_switch_site: SiteBlockInactive | None
    vendor: Vendor | None = None


class SuperPopSwitchBlockProvisioning(SuperPopSwitchBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A Super PoP switch that's being provisioned. See :class:`SuperPopSwitchBlock`."""

    super_pop_switch_fqdn: str | None = None
    super_pop_switch_ts_port: PortNumber | None = None
    super_pop_switch_mgmt_ipv4_address: IPv4AddressType | None = None
    super_pop_switch_site: SiteBlockProvisioning | None
    vendor: Vendor | None = None


class SuperPopSwitchBlock(SuperPopSwitchBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A Super PoP switch that's currently deployed in the network."""

    #:  Super PoP switch FQDN.
    super_pop_switch_fqdn: str
    #:  The port of the terminal server that this Super PoP switch is connected to. Used to offer out of band access.
    super_pop_switch_ts_port: PortNumber
    #:  The IPv4 management address of the Super PoP switch.
    super_pop_switch_mgmt_ipv4_address: IPv4AddressType
    #:  The :class:`Site` that this Super PoP switch resides in. Both physically and computationally.
    super_pop_switch_site: SiteBlock
    #:  The vendor of a Super PoP switch. Defaults to Juniper.
    vendor: Vendor = Vendor.JUNIPER
